@@ -35,6 +35,7 @@
 #include <errno.h>
 #include "gettext.h"
 #include "version.c"
+#include <linux/fs.h>
 #define _(STRING) gettext(STRING)
 //#define PACKAGE "partclone"
 
@@ -60,8 +61,8 @@ FILE* msg = NULL;
 
 /**
  * options - 
- * usage	    - print message "how to use this"
- * parse_options    - get parameter from agrc, argv
+ * usage		    - print message "how to use this"
+ * parse_options	    - get parameter from agrc, argv
  */
 extern void usage(void)
 {
@@ -71,11 +72,13 @@ extern void usage(void)
         "    -o,  --output FILE      Output FILE\n"
 	"    -O   --overwrite FILE   Output FILE, overwriting if exists\n"
 	"    -s,  --source FILE      Source FILE\n"
+#ifndef	RESTORE
         "    -c,  --clone            Save to the special image format\n"
         "    -r,  --restore          Restore from the special image format\n"
 	"    -b,  --dd-mode          Save to sector-to-sector format\n"
-        "    -dX, --debug=X          Set the debug level to X = [0|1|2]\n"
         "    -R,  --rescue           Continue after disk read errors\n"
+#endif
+        "    -dX, --debug=X          Set the debug level to X = [0|1|2]\n"
         "    -C,  --no_check         Don't check device size and free space\n"
 #ifdef HAVE_LIBNCURSESW
         "    -N,  --ncurses          Using Ncurses User Interface\n"
@@ -115,7 +118,10 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
     opt->debug = 0;
     opt->rescue = 0;
     opt->check = 1;
-
+#ifdef RESTORE
+    opt->restore++;
+    mode++;
+#endif
     while ((c = getopt_long(argc, argv, sopt, lopt, NULL)) != (char)-1) {
             switch (c) {
             case 's': 
@@ -186,6 +192,11 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
     }
     //printf("debug %i\n", opt->debug);
         
+    if ((opt->target == NULL) && (opt->source == NULL)){
+	fprintf(stderr, "There is no image name. or --help get more info.\n");
+	exit(0);
+    }
+
     if (opt->target == NULL) {
 	//fprintf(stderr, "You use specify output file like stdout. or --help get more info.\n");
 	opt->target = "-";
@@ -364,6 +375,7 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
     extern p_dialog_mesg m_dialog;
     char tmp_str[128];
 	
+    vsprintf(tmp_str, fmt, args);
     if (opt.ncurses) {
 #ifdef HAVE_LIBNCURSESW
 	setlocale(LC_ALL, "");
@@ -374,7 +386,8 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
 		wattron(log_win, A_STANDOUT);
 	    }
 
-	    vwprintw(log_win, fmt, args);
+	    wprintw(log_win, tmp_str);
+
 	    
 	    if(log_exit){
 		wattroff(log_win, A_STANDOUT);
@@ -387,21 +400,21 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
     } else if (opt.dialog) {
 	/// write log to stderr if log_stderr is true
 	if((log_stderr) && (log_level <= debug)){
-	    vsprintf(tmp_str, fmt, args);
+	    //vsprintf(tmp_str, fmt, args);
 	    strncat(m_dialog.data, tmp_str, strlen(tmp_str));
 	    fprintf(stderr, "XXX\n%i\n%s\nXXX\n", m_dialog.percent, m_dialog.data);
 	}
     } else {
 	/// write log to stderr if log_stderr is true
 	if((log_stderr == 1) && (log_level <= debug)){
-	    vsprintf(tmp_str, fmt, args);
+	    //vsprintf(tmp_str, fmt, args);
 	    fprintf(stderr, "%s", tmp_str);
 	}
     }
 
     /// write log to logfile if debug is true
     if(log_level <= debug){
-	vsprintf(tmp_str, fmt, args);
+	//vsprintf(tmp_str, fmt, args);
 	fprintf(msg, "%s", tmp_str);
 	//if (errno != 0)
 	//    fprintf(msg, "%s(%i), ", strerror(errno), errno);
@@ -412,7 +425,7 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
     fflush(msg);
 
     /// exit if lexit true
-    if (log_exit){
+    if ((!opt.force) && (log_exit)){
 	close_ncurses();
 	fprintf(stderr, "Partclone fail, please check /var/log/partclone.log !\n");
     	exit(1);
@@ -477,7 +490,43 @@ extern int check_size(int* ret, unsigned long long size){
 
 }
 
-/// chech free space 
+/// get partition size
+extern unsigned long long get_partition_size(int* ret){
+
+    unsigned long long dest_size = 0;
+    unsigned long dest_block;
+    struct stat stat;
+    int debug = 1;
+
+    if (!fstat(*ret, &stat)) {
+        if (S_ISFIFO(stat.st_mode)){
+            dest_size = 0;
+	} else if (S_ISREG(stat.st_mode)){
+	    dest_size = stat.st_size;
+	} else {
+
+#ifdef BLKGETSIZE64
+	    if (ioctl(*ret, BLKGETSIZE64, &dest_size) < 0) {
+		log_mesg(0, 0, 0, debug, "get device size error\n");
+	    }
+#endif
+
+#ifdef BLKGETSIZE
+	    if (ioctl(*ret, BLKGETSIZE, &dest_block) >= 0) {
+		dest_size = (unsigned long long)(dest_block * 512);
+	    }
+#endif
+	}
+    } else {
+	log_mesg(0, 0, 0, debug, "fstat size error\n");
+    
+    }
+
+    return dest_size;
+
+}
+
+/// check free space 
 extern void check_free_space(int* ret, unsigned long long size){
 
     unsigned long long dest_size;
@@ -702,10 +751,13 @@ extern int io_all(int *fd, char *buf, int count, int do_write, cmd_opt* opt)
 
 	if (i < 0) {
 	    if (errno != EAGAIN && errno != EINTR){
-		log_mesg(0, 1, 1, debug, "%s: errno = %i\n",__func__, errno);
+		log_mesg(0, 1, 1, debug, "%s: errno = %s(%i)\n",__func__, strerror(errno), errno);
                 return -1;
 	    }
-	    log_mesg(0, 1, 1, debug, "%s: errno = %i\n",__func__, errno);
+	    log_mesg(0, 1, 1, debug, "%s: errno = %s(%i)\n",__func__, strerror(errno), errno);
+	} else if (i == 0){
+	    log_mesg(2, 0, 0, debug, "%s: errno = %s(%i)\n",__func__, strerror(errno), errno);
+	    return 0;
         } else {
 	    count -= i;
 	    buf = i + (char *) buf;
@@ -809,24 +861,24 @@ extern void print_image_hdr_info(image_head image_hdr, cmd_opt opt){
     unsigned long long dev_size = image_hdr.device_size;
     int debug = opt.debug;
 
-	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
-	//log_mesg(0, 0, 0, "%s v%s \n", EXEC_NAME, VERSION);
-	log_mesg(0, 0, 1, debug, _("Partclone v%s (%s) www.partclone.org, partclone.nchc.org.tw\n"), VERSION, svn_version);
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+    //log_mesg(0, 0, 0, "%s v%s \n", EXEC_NAME, VERSION);
+    log_mesg(0, 0, 1, debug, _("Partclone v%s (%s) www.partclone.org, partclone.nchc.org.tw\n"), VERSION, svn_version);
     if (opt.clone)
-		log_mesg(0, 0, 1, debug, _("Starting clone device (%s) to image (%s)\n"), opt.source, opt.target);	
-	else if(opt.restore)
-		log_mesg(0, 0, 1, debug, _("Starting restore image (%s) to device (%s)\n"), opt.source, opt.target);
-	else if(opt.dd)
-		log_mesg(0, 0, 1, debug, _("Starting back up device(%s) to device(%s)\n"), opt.source, opt.target);
-	else
-		log_mesg(0, 0, 1, debug, "unknow mode\n");
-	log_mesg(0, 0, 1, debug, _("File system: %s\n"), image_hdr.fs);
-	log_mesg(0, 0, 1, debug, _("Device size: %lli MB\n"), print_size((total*block_s), MBYTE));
-	log_mesg(0, 0, 1, debug, _("Space in use: %lli MB\n"), print_size((used*block_s), MBYTE));
-	log_mesg(0, 0, 1, debug, _("Block size: %i Byte\n"), block_s);
-	log_mesg(0, 0, 1, debug, _("Used block count: %lli\n"), used);
+	log_mesg(0, 0, 1, debug, _("Starting clone device (%s) to image (%s)\n"), opt.source, opt.target);	
+    else if(opt.restore)
+	log_mesg(0, 0, 1, debug, _("Starting restore image (%s) to device (%s)\n"), opt.source, opt.target);
+    else if(opt.dd)
+	log_mesg(0, 0, 1, debug, _("Starting back up device(%s) to device(%s)\n"), opt.source, opt.target);
+    else
+	log_mesg(0, 0, 1, debug, "unknow mode\n");
+    log_mesg(0, 0, 1, debug, _("File system: %s\n"), image_hdr.fs);
+    log_mesg(0, 0, 1, debug, _("Device size: %lli MB\n"), print_size((total*block_s), MBYTE));
+    log_mesg(0, 0, 1, debug, _("Space in use: %lli MB\n"), print_size((used*block_s), MBYTE));
+    log_mesg(0, 0, 1, debug, _("Block size: %i Byte\n"), block_s);
+    log_mesg(0, 0, 1, debug, _("Used block count: %lli\n"), used);
 }
 
 /// print finish message
@@ -846,3 +898,37 @@ extern void print_finish_info(cmd_opt opt){
 	    
 }
 
+/// block_count
+static unsigned long long block_count(unsigned long long partition_size, int block_size){
+    unsigned long long blocks;
+    int sectors;
+    blocks  = partition_size / block_size;
+    sectors = partition_size % block_size;
+    if (sectors)
+	return (blocks+1);
+    else
+	return blocks;
+
+}
+
+/// initial dd hdr
+extern void initial_dd_hdr(int ret, image_head* image_hdr){
+
+    memset(image_hdr ,(int )NULL, sizeof(image_head));
+    memcpy(image_hdr->fs, raw_MAGIC, FS_MAGIC_SIZE);
+    image_hdr->block_size  = 512;
+    image_hdr->device_size = get_partition_size(&ret);
+    image_hdr->totalblock  = block_count(image_hdr->device_size, image_hdr->block_size);
+    image_hdr->usedblocks  = image_hdr->totalblock;
+}
+
+/// initial bitmap
+extern void dd_bitmap(image_head image_hdr, char* bitmap){
+
+    int block;
+
+    /// initial image bitmap as 1 (all block are used)
+    for(block = 0; block < image_hdr.totalblock; block++)
+	bitmap[block] = 1; 
+
+}
